/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.ow2.petals.cli.extension.command.monitoring.mo.Constants.QUERY_RESULT_OUTPUT_FORMAT_S_D;
import static org.ow2.petals.cli.extension.command.monitoring.mo.Constants.QUERY_RESULT_OUTPUT_FORMAT_S_S;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvokationKey;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvokationStatus;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;

/**
 * The monitoring sub-function about service provider invokation response times
 * of a component.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ServiceProviderInvokationsResponseTimes implements MonitoringSubFunction {

    private final static String SUBFCT_NAME = "service-provider-invokations-response-times";

    private final static String SUBFCT_DESCRIPTION = "Response times of service provider invokations";

    /**
     * The option to get the component name
     */
    public static final String COMPONENT_NAME_LONG_OPTION = "component-name";

    public static final String COMPONENT_NAME_SHORT_OPTION = "n";

    private static final Option COMPONENT_NAME_OPTION = OptionBuilder.isRequired(true).hasArgs(1)
            .withArgName("component-name").withDescription("The identifier of the component.")
            .withLongOpt(COMPONENT_NAME_LONG_OPTION).create(COMPONENT_NAME_SHORT_OPTION);

    /**
     * The option to get the list of invoked operation for which at least one
     * request was received.
     */
    public static final String LIST_OPERATIONS_LONG_OPTION = "list-operations";

    private static final Option LIST_OPERATIONS_OPTION = OptionBuilder.isRequired(false)
            .hasArg(false)
            .withDescription("List the operations for which at least one request was received.")
            .withLongOpt(LIST_OPERATIONS_LONG_OPTION).create();

    /**
     * The option to query invoked operations of service provider invokations
     */
    public static final String QUERY_OPERATIONS_LONG_OPTION = "query-operations";

    private static final Option QUERY_OPERATIONS_OPTION = OptionBuilder.isRequired(false)
            .hasOptionalArgs(2).withDescription("Query about operations")
            .withLongOpt(QUERY_OPERATIONS_LONG_OPTION).create();

    /**
     * The option to get the complete table of values
     */
    public static final String TABLE_SHORT_OPTION = "t";

    private static final Option TABLE_OPTION = OptionBuilder.isRequired(false)
            .withDescription("Get the complete table of values").create(TABLE_SHORT_OPTION);

    /**
     * Query attribute to get index name
     */
    public static final String QUERY_ATTR_NAME = "NAME";

    /**
     * Query attribute to get the min response time of service provider
     * invokations successfuly processed.
     */
    public static final String QUERY_ATTR_SUCCEEDED_MIN = "SUCCEEDED_MIN";

    /**
     * Query attribute to get the average response time of service provider
     * invokations successfuly processed.
     */
    public static final String QUERY_ATTR_SUCCEEDED_AVG = "SUCCEEDED_AVG";

    /**
     * Query attribute to get the max response time of service provider
     * invokationss successfuly processed.
     */
    public static final String QUERY_ATTR_SUCCEEDED_MAX = "SUCCEEDED_MAX";

    /**
     * Query attribute to get the 10-percentile response time of service
     * provider invokations successfuly processed.
     */
    public static final String QUERY_ATTR_SUCCEEDED_10P = "SUCCEEDED_10P";

    /**
     * Query attribute to get the 50-percentile response time of service
     * provider invokations successfuly processed.
     */
    public static final String QUERY_ATTR_SUCCEEDED_50P = "SUCCEEDED_50P";

    /**
     * Query attribute to get the 90-percentile response time of service
     * provider invokations successfuly processed.
     */
    public static final String QUERY_ATTR_SUCCEEDED_90P = "SUCCEEDED_90P";

    /**
     * Query attribute to get the min response time of service provider
     * invokations faulty processed.
     */
    public static final String QUERY_ATTR_FAULT_MIN = "FAULT_MIN";

    /**
     * Query attribute to get the average response time of service provider
     * invokations faulty processed.
     */
    public static final String QUERY_ATTR_FAULT_AVG = "FAULT_AVG";

    /**
     * Query attribute to get the max response time of service provider
     * invokations faulty processed.
     */
    public static final String QUERY_ATTR_FAULT_MAX = "FAULT_MAX";

    /**
     * Query attribute to get the 10-percentile response time of service
     * provider invokations faulty processed.
     */
    public static final String QUERY_ATTR_FAULT_10P = "FAULT_10P";

    /**
     * Query attribute to get the 50-percentile response time of service
     * provider invokations faulty processed.
     */
    public static final String QUERY_ATTR_FAULT_50P = "FAULT_50P";

    /**
     * Query attribute to get the 90-percentile response time of service
     * provider invokations faulty processed.
     */
    public static final String QUERY_ATTR_FAULT_90P = "FAULT_90P";

    /**
     * Query attribute to get the min response time of service provider
     * invokations for which an error occurs
     */
    public static final String QUERY_ATTR_ERROR_MIN = "ERROR_MIN";

    /**
     * Query attribute to get the average response time of service provider
     * invokations for which an error occurs
     */
    public static final String QUERY_ATTR_ERROR_AVG = "ERROR_AVG";

    /**
     * Query attribute to get the max response time of service provider
     * invokations for which an error occurs
     */
    public static final String QUERY_ATTR_ERROR_MAX = "ERROR_MAX";

    /**
     * Query attribute to get the 10-percentile response time of service
     * provider invokations for which an error occurs
     */
    public static final String QUERY_ATTR_ERROR_10P = "ERROR_10P";

    /**
     * Query attribute to get the 50-percentile response time of service
     * provider invokations for which an error occurs
     */
    public static final String QUERY_ATTR_ERROR_50P = "ERROR_50P";

    /**
     * Query attribute to get the 90-percentile response time of service
     * provider invokations for which an error occurs
     */
    public static final String QUERY_ATTR_ERROR_90P = "ERROR_90P";

    private final static String HEADER_INTERFACE_NAME = "Interface";

    private final static String HEADER_SERVICE_NAME = "Service";

    private final static String HEADER_OPERATION = "Operation";

    private final static String HEADER_MEP = "MEP";

    private final static String HEADER_EXEC_STATUS = "Execution status";

    private final static String HEADER_VALUE_MIN = "Min";

    private final static String HEADER_VALUE_AVG = "Avg";

    private final static String HEADER_VALUE_MAX = "Max";

    private final static String HEADER_VALUE_10P = "10P";

    private final static String HEADER_VALUE_50P = "50P";

    private final static String HEADER_VALUE_90P = "90P";

    /**
     * The sub-function options
     */
    private Options options;

    /**
     * The shell in which the sub-function is running
     */
    private Shell shell;

    public ServiceProviderInvokationsResponseTimes() {
        super();
        this.createOptions();
    }

    // TODO: Options declaration must be reviewed to match:
    // -n <component-name>
    // [--list-operations] |
    // [--query-operations [NAME] | [SUCCEEDED|FAULT|ERROR <index>]] |
    // [-t]
    private void createOptions() {
        this.options = new Options();
        this.options.addOption(COMPONENT_NAME_OPTION);
        this.options.addOption(LIST_OPERATIONS_OPTION);
        this.options.addOption(QUERY_OPERATIONS_OPTION);
        this.options.addOption(TABLE_OPTION);
    }

    @Override
    public Options getOptions() {
        return this.options;
    }

    @Override
    public void resetOptions() {
        this.createOptions();
    }

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void setShell(final Shell shell) {
        this.shell = shell;
    }

    @Override
    public Shell getShell() {
        return this.shell;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionInvalidException,
            MonitoringSubFunctionException {

        final CommandLineParser parser = new PosixParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            // No extra option is accepted
            if (cli.getArgs().length > 0) {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            if (cli.hasOption(LIST_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(QUERY_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listOperations(componentIdentifier, false, null);

            } else if (!cli.hasOption(LIST_OPERATIONS_LONG_OPTION)
                    && cli.hasOption(QUERY_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                final String[] queryArgs = cli.getOptionValues(QUERY_OPERATIONS_LONG_OPTION);
                if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null
                        || queryArgs[0].isEmpty()) {
                    throw new MonitoringSubFunctionMissingArgumentException(this,
                            QUERY_OPERATIONS_OPTION);
                } else if (!QUERY_ATTR_NAME.equals(queryArgs[0])
                        && !QUERY_ATTR_SUCCEEDED_MIN.equals(queryArgs[0])
                        && !QUERY_ATTR_SUCCEEDED_AVG.equals(queryArgs[0])
                        && !QUERY_ATTR_SUCCEEDED_MAX.equals(queryArgs[0])
                        && !QUERY_ATTR_SUCCEEDED_10P.equals(queryArgs[0])
                        && !QUERY_ATTR_SUCCEEDED_50P.equals(queryArgs[0])
                        && !QUERY_ATTR_SUCCEEDED_90P.equals(queryArgs[0])
                        && !QUERY_ATTR_FAULT_MIN.equals(queryArgs[0])
                        && !QUERY_ATTR_FAULT_AVG.equals(queryArgs[0])
                        && !QUERY_ATTR_FAULT_MAX.equals(queryArgs[0])
                        && !QUERY_ATTR_FAULT_10P.equals(queryArgs[0])
                        && !QUERY_ATTR_FAULT_50P.equals(queryArgs[0])
                        && !QUERY_ATTR_FAULT_90P.equals(queryArgs[0])
                        && !QUERY_ATTR_ERROR_MIN.equals(queryArgs[0])
                        && !QUERY_ATTR_ERROR_AVG.equals(queryArgs[0])
                        && !QUERY_ATTR_ERROR_MAX.equals(queryArgs[0])
                        && !QUERY_ATTR_ERROR_10P.equals(queryArgs[0])
                        && !QUERY_ATTR_ERROR_50P.equals(queryArgs[0])
                        && !QUERY_ATTR_ERROR_90P.equals(queryArgs[0])) {
                    throw new MonitoringSubFunctionInvalidArgumentException(this,
                            QUERY_OPERATIONS_OPTION, queryArgs[0]);
                } else {
                    this.listOperations(componentIdentifier, true,
                        cli.getOptionValues(QUERY_OPERATIONS_LONG_OPTION));
                }

            } else if (!cli.hasOption(LIST_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(QUERY_OPERATIONS_LONG_OPTION)
                    && cli.hasOption(TABLE_SHORT_OPTION)) {

                this.fullTable(componentIdentifier);

            } else {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        }
    }

    /**
     * List operations for which a metric about web service requests is
     * available
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the metric value.
     */
    private void listOperations(final String componentIdentifier, final boolean hasQuery,
            final String[] queryArgs) throws MonitoringSubFunctionException {

        if (hasQuery) {
            this.query(componentIdentifier, queryArgs, QUERY_OPERATIONS_OPTION);
        } else {
            try {
                final JMXClient client = JMXClientConnection.getJMXClient();
                final Map<ServiceProviderInvokationKey, Long[]> values = client
                        .getComponentMonitoringServiceClient(componentIdentifier)
                        .getServiceProviderInvokationsResponseTimes();

                if (values != null) {
                    final List<String> operations = new ArrayList<String>();
                    for (final ServiceProviderInvokationKey key : values.keySet()) {
                        final String operation = ServiceProviderInvokationsResponseTimes
                                .getOperationName(key);
                        if (!operations.contains(operation)) {
                            operations.add(operation);
                        }
                    }

                    for (final String operation : operations) {
                        this.shell.getPrintStream().println(operation);
                    }
                }

            } catch (final ConnectionErrorException e) {
                throw new MonitoringSubFunctionException(this, e);
            } catch (final ComponentMonitoringServiceErrorException e) {
                throw new MonitoringSubFunctionException(this, e);
            } catch (final ComponentMonitoringServiceDoesNotExistException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier));
            }
        }
    }

    private static String getOperationName(final ServiceProviderInvokationKey key) {
        return key.getOperation() + "@" + key.getServiceName() + "@" + key.getInterfaceName();
    }

    private void query(final String componentIdentifier, final String[] queryArgs,
            final Option queryOption) throws MonitoringSubFunctionException {

        if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null
                || queryArgs[0].isEmpty()) {
            throw new MonitoringSubFunctionMissingArgumentException(this, queryOption);
        } else {
            try {
                final JMXClient client = JMXClientConnection.getJMXClient();
                if (QUERY_ATTR_NAME.equals(queryArgs[0])) {
                    final Map<ServiceProviderInvokationKey, Long[]> values = client
                            .getComponentMonitoringServiceClient(componentIdentifier)
                            .getServiceProviderInvokationsResponseTimes();

                    if (values != null) {
                        final List<String> operations = new ArrayList<String>();
                        for (final ServiceProviderInvokationKey key : values.keySet()) {
                            final String operation = ServiceProviderInvokationsResponseTimes
                                    .getOperationName(key);
                            if (!operations.contains(operation)) {
                                operations.add(operation);
                            }
                        }

                        for (final String operation : operations) {
                            this.shell.getPrintStream().println(
                                    String.format(QUERY_RESULT_OUTPUT_FORMAT_S_S, operation,
                                            operation));
                        }
                    }
                } else if (QUERY_ATTR_SUCCEEDED_MIN.equals(queryArgs[0])
                        || QUERY_ATTR_SUCCEEDED_AVG.equals(queryArgs[0])
                        || QUERY_ATTR_SUCCEEDED_MAX.equals(queryArgs[0])
                        || QUERY_ATTR_SUCCEEDED_10P.equals(queryArgs[0])
                        || QUERY_ATTR_SUCCEEDED_50P.equals(queryArgs[0])
                        || QUERY_ATTR_SUCCEEDED_90P.equals(queryArgs[0])
                        || QUERY_ATTR_FAULT_MIN.equals(queryArgs[0])
                        || QUERY_ATTR_FAULT_AVG.equals(queryArgs[0])
                        || QUERY_ATTR_FAULT_MAX.equals(queryArgs[0])
                        || QUERY_ATTR_FAULT_10P.equals(queryArgs[0])
                        || QUERY_ATTR_FAULT_50P.equals(queryArgs[0])
                        || QUERY_ATTR_FAULT_90P.equals(queryArgs[0])
                        || QUERY_ATTR_ERROR_MIN.equals(queryArgs[0])
                        || QUERY_ATTR_ERROR_AVG.equals(queryArgs[0])
                        || QUERY_ATTR_ERROR_MAX.equals(queryArgs[0])
                        || QUERY_ATTR_ERROR_10P.equals(queryArgs[0])
                        || QUERY_ATTR_ERROR_50P.equals(queryArgs[0])
                        || QUERY_ATTR_ERROR_90P.equals(queryArgs[0])) {
                    final String index = queryArgs.length == 2 ? queryArgs[1] : null;
                    final Map<ServiceProviderInvokationKey, Long[]> values = client
                            .getComponentMonitoringServiceClient(componentIdentifier)
                            .getServiceProviderInvokationsResponseTimesGroupedByItfSvcOpAndExecStatus();
                    if (values != null) {
                        if (index != null) {

                            Long value = null;
                            for (final Map.Entry<ServiceProviderInvokationKey, Long[]> entry : values
                                    .entrySet()) {

                                final ServiceProviderInvokationKey key = entry.getKey();
                                final String operationName = ServiceProviderInvokationsResponseTimes
                                        .getOperationName(key);
                                final ServiceProviderInvokationStatus execStatus = key
                                        .getExecStatus();
                                if (index.equals(operationName)
                                        && ((ServiceProviderInvokationStatus.SUCCEEDED == execStatus
                                                && (QUERY_ATTR_SUCCEEDED_MIN.equals(queryArgs[0])
                                                || QUERY_ATTR_SUCCEEDED_AVG.equals(queryArgs[0])
                                                || QUERY_ATTR_SUCCEEDED_MAX.equals(queryArgs[0])
                                                || QUERY_ATTR_SUCCEEDED_10P.equals(queryArgs[0])
                                                || QUERY_ATTR_SUCCEEDED_50P.equals(queryArgs[0])
                                                || QUERY_ATTR_SUCCEEDED_90P.equals(queryArgs[0])))
                                                || (ServiceProviderInvokationStatus.FAULT == execStatus
                                                && (QUERY_ATTR_FAULT_MIN.equals(queryArgs[0])
                                                || QUERY_ATTR_FAULT_AVG.equals(queryArgs[0])
                                                || QUERY_ATTR_FAULT_MAX.equals(queryArgs[0])
                                                || QUERY_ATTR_FAULT_10P.equals(queryArgs[0])
                                                || QUERY_ATTR_FAULT_50P.equals(queryArgs[0])
                                                || QUERY_ATTR_FAULT_90P.equals(queryArgs[0])))
                                                || (ServiceProviderInvokationStatus.ERROR == execStatus
                                                && (QUERY_ATTR_ERROR_MIN.equals(queryArgs[0])
                                                || QUERY_ATTR_ERROR_AVG.equals(queryArgs[0])
                                                || QUERY_ATTR_ERROR_MAX.equals(queryArgs[0])
                                                || QUERY_ATTR_ERROR_10P.equals(queryArgs[0])
                                                || QUERY_ATTR_ERROR_50P.equals(queryArgs[0])
                                                || QUERY_ATTR_ERROR_90P.equals(queryArgs[0]))) )) {

                                    final Long[] responseTimes = entry.getValue();
                                    value = this.getResponseTimeValueAccordingToQueryArgs(
                                            responseTimes, queryArgs, queryOption);
                                    break;
                                }
                            }

                            if (value == null) {
                                // Nothing is printed if no value.
                                this.shell.getPrintStream().println(String.format("%n"));
                            } else {
                                this.shell.getPrintStream().println(String.format("%s", value));
                            }
                        } else {

                            for (final Map.Entry<ServiceProviderInvokationKey, Long[]> entry : values
                                    .entrySet()) {
                                final ServiceProviderInvokationKey key = entry.getKey();
                                final ServiceProviderInvokationStatus execStatus = key
                                        .getExecStatus();
                                final Long[] responseTimes = entry.getValue();
                                if ((ServiceProviderInvokationStatus.SUCCEEDED == execStatus && (QUERY_ATTR_SUCCEEDED_MIN
                                        .equals(queryArgs[0])
                                        || QUERY_ATTR_SUCCEEDED_AVG.equals(queryArgs[0])
                                        || QUERY_ATTR_SUCCEEDED_MAX.equals(queryArgs[0])
                                        || QUERY_ATTR_SUCCEEDED_10P.equals(queryArgs[0])
                                        || QUERY_ATTR_SUCCEEDED_50P.equals(queryArgs[0]) || QUERY_ATTR_SUCCEEDED_90P
                                            .equals(queryArgs[0])))
                                        || (ServiceProviderInvokationStatus.FAULT == execStatus && (QUERY_ATTR_FAULT_MIN
                                                .equals(queryArgs[0])
                                                || QUERY_ATTR_FAULT_AVG.equals(queryArgs[0])
                                                || QUERY_ATTR_FAULT_MAX.equals(queryArgs[0])
                                                || QUERY_ATTR_FAULT_10P.equals(queryArgs[0])
                                                || QUERY_ATTR_FAULT_50P.equals(queryArgs[0]) || QUERY_ATTR_FAULT_90P
                                                    .equals(queryArgs[0])))
                                        || (ServiceProviderInvokationStatus.ERROR == execStatus && (QUERY_ATTR_ERROR_MIN
                                                .equals(queryArgs[0])
                                                || QUERY_ATTR_ERROR_AVG.equals(queryArgs[0])
                                                || QUERY_ATTR_ERROR_MAX.equals(queryArgs[0])
                                                || QUERY_ATTR_ERROR_10P.equals(queryArgs[0])
                                                || QUERY_ATTR_ERROR_50P.equals(queryArgs[0]) || QUERY_ATTR_ERROR_90P
                                                    .equals(queryArgs[0])))) {

                                    final Long value = this
                                            .getResponseTimeValueAccordingToQueryArgs(
                                                    responseTimes, queryArgs, queryOption);
                                    this.shell.getPrintStream().println(
                                            String.format(QUERY_RESULT_OUTPUT_FORMAT_S_D,
                                                    ServiceProviderInvokationsResponseTimes
                                                            .getOperationName(key), value));
                                }
                            }
                        }
                    }
                } else {
                    throw new MonitoringSubFunctionInvalidArgumentException(this, queryOption,
                            queryArgs[0]);
                }
            } catch (final ConnectionErrorException e) {
                throw new MonitoringSubFunctionException(this, e);
            } catch (final ComponentMonitoringServiceErrorException e) {
                throw new MonitoringSubFunctionException(this, e);
            } catch (final ComponentMonitoringServiceDoesNotExistException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier));
            }
        }
    }

    private Long getResponseTimeValueAccordingToQueryArgs(final Long[] responseTimes,
            final String[] queryArgs, final Option queryOption)
            throws MonitoringSubFunctionInvalidArgumentException {
        if (QUERY_ATTR_SUCCEEDED_MIN.equals(queryArgs[0])
                || QUERY_ATTR_FAULT_MIN.equals(queryArgs[0])
                || QUERY_ATTR_ERROR_MIN.equals(queryArgs[0])) {
            return responseTimes[2];
        } else if (QUERY_ATTR_SUCCEEDED_AVG.equals(queryArgs[0])
                || QUERY_ATTR_FAULT_AVG.equals(queryArgs[0])
                || QUERY_ATTR_ERROR_AVG.equals(queryArgs[0])) {
            return responseTimes[1];
        } else if (QUERY_ATTR_SUCCEEDED_MAX.equals(queryArgs[0])
                || QUERY_ATTR_FAULT_MAX.equals(queryArgs[0])
                || QUERY_ATTR_ERROR_MAX.equals(queryArgs[0])) {
            return responseTimes[0];
        } else if (QUERY_ATTR_SUCCEEDED_10P.equals(queryArgs[0])
                || QUERY_ATTR_FAULT_10P.equals(queryArgs[0])
                || QUERY_ATTR_ERROR_10P.equals(queryArgs[0])) {
            return responseTimes[3];
        } else if (QUERY_ATTR_SUCCEEDED_50P.equals(queryArgs[0])
                || QUERY_ATTR_FAULT_50P.equals(queryArgs[0])
                || QUERY_ATTR_ERROR_50P.equals(queryArgs[0])) {
            return responseTimes[4];
        } else if (QUERY_ATTR_SUCCEEDED_90P.equals(queryArgs[0])
                || QUERY_ATTR_FAULT_90P.equals(queryArgs[0])
                || QUERY_ATTR_ERROR_90P.equals(queryArgs[0])) {
            return responseTimes[5];
        } else {
            throw new MonitoringSubFunctionInvalidArgumentException(this, queryOption, queryArgs[0]);
        }
    }

    /**
     * Print the full table
     * 
     * @param componentIdentifier
     *            The identifier of the component
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     */
    private void fullTable(final String componentIdentifier) throws MonitoringSubFunctionException {

        try {
            final JMXClient client = JMXClientConnection.getJMXClient();
            final Map<ServiceProviderInvokationKey, Long[]> values = client
                    .getComponentMonitoringServiceClient(componentIdentifier)
                    .getServiceProviderInvokationsResponseTimes();

            int maxSizeItf = 0;
            int maxSizeSvc = 0;
            int maxSizeOp = 0;
            int maxSizeMep = 0;
            int maxSizeExecStatus = 0;
            int maxSizeValueMin = 0;
            int maxSizeValueAvg = 0;
            int maxSizeValueMax = 0;
            int maxSizeValue10P = 0;
            int maxSizeValue50P = 0;
            int maxSizeValue90P = 0;
            if (values != null) {
                for (final Map.Entry<ServiceProviderInvokationKey, Long[]> entry : values
                        .entrySet()) {
                    final ServiceProviderInvokationKey key = entry.getKey();
                    final String interfaceName = key.getInterfaceName();
                    final String serviceName = key.getServiceName();
                    final String operation = key.getOperation();
                    final String mep = key.getMep();
                    final String execStatus = key.getExecStatus().toString();
                    maxSizeItf = Math.max(maxSizeItf,
                            interfaceName == null ? 0 : interfaceName.length());
                    maxSizeSvc = Math.max(maxSizeSvc,
                            serviceName == null ? 0 : serviceName.length());
                    maxSizeOp = Math.max(maxSizeOp, operation == null ? 0 : operation.length());
                    maxSizeMep = Math.max(maxSizeMep,
 mep == null ? 0 : mep.length());
                    maxSizeExecStatus = Math.max(maxSizeExecStatus, execStatus == null ? 0
                            : execStatus.length());
                    maxSizeValueMin = Math.max(maxSizeValueMin, entry.getValue()[2].toString()
                            .length());
                    maxSizeValueAvg = Math.max(maxSizeValueAvg, entry.getValue()[1].toString()
                            .length());
                    maxSizeValueMax = Math.max(maxSizeValueMax, entry.getValue()[0].toString()
                            .length());
                    maxSizeValue10P = Math.max(maxSizeValue10P, entry.getValue()[3].toString()
                            .length());
                    maxSizeValue50P = Math.max(maxSizeValue50P, entry.getValue()[4].toString()
                            .length());
                    maxSizeValue90P = Math.max(maxSizeValue90P, entry.getValue()[5].toString()
                            .length());
                }
            }

            maxSizeItf = Math.max(maxSizeItf, HEADER_INTERFACE_NAME.length());
            maxSizeSvc = Math.max(maxSizeSvc, HEADER_SERVICE_NAME.length());
            maxSizeOp = Math.max(maxSizeOp, HEADER_OPERATION.length());
            maxSizeMep = Math.max(maxSizeMep, HEADER_MEP.length());
            maxSizeExecStatus = Math.max(maxSizeExecStatus, HEADER_EXEC_STATUS.length());
            maxSizeValueMin = Math.max(maxSizeValueMin, HEADER_VALUE_MIN.length());
            maxSizeValueAvg = Math.max(maxSizeValueAvg, HEADER_VALUE_AVG.length());
            maxSizeValueMax = Math.max(maxSizeValueMax, HEADER_VALUE_MAX.length());
            maxSizeValue10P = Math.max(maxSizeValue10P, HEADER_VALUE_10P.length());
            maxSizeValue50P = Math.max(maxSizeValue50P, HEADER_VALUE_50P.length());
            maxSizeValue90P = Math.max(maxSizeValue90P, HEADER_VALUE_90P.length());

            final String rowHeaderFormat = "| %-" + maxSizeItf + "s | %-" + maxSizeSvc + "s | %-"
                    + maxSizeOp + "s | %-"
                    + maxSizeMep + "s | %-" + maxSizeExecStatus + "s || %-" + maxSizeValueMin
                    + "s | %-" + maxSizeValueAvg + "s | %-" + maxSizeValueMax + "s | %-"
                    + maxSizeValue10P + "s | %-" + maxSizeValue50P + "s | %-" + maxSizeValue90P
                    + "s |";
            final String header = String.format(rowHeaderFormat, HEADER_INTERFACE_NAME,
                    HEADER_SERVICE_NAME, HEADER_OPERATION, HEADER_MEP, HEADER_EXEC_STATUS,
                    HEADER_VALUE_MIN, HEADER_VALUE_AVG, HEADER_VALUE_MAX, HEADER_VALUE_10P,
                    HEADER_VALUE_50P, HEADER_VALUE_90P);
            this.shell.getPrintStream().println(header);
            for (int i = 0; i < header.length(); i++) {
                this.shell.getPrintStream().append('-');
            }
            this.shell.getPrintStream().append('\n');

            if (values != null) {
                final String rowFormat = "| %-" + maxSizeItf + "s | %-" + maxSizeSvc + "s | %-"
                        + maxSizeOp + "s | %-"
                        + maxSizeMep + "s | %-" + maxSizeExecStatus + "s || %" + maxSizeValueMin
                        + "d | %" + maxSizeValueAvg + "d | %" + maxSizeValueMax + "d | %"
                        + maxSizeValue10P + "d | %" + maxSizeValue50P + "d | %" + maxSizeValue90P
                        + "d |";
                for (final Map.Entry<ServiceProviderInvokationKey, Long[]> entry : values
                        .entrySet()) {

                    final ServiceProviderInvokationKey key = entry.getKey();
                    final String interfaceName = key.getInterfaceName();
                    final String serviceName = key.getServiceName();
                    final String operation = key.getOperation();
                    final String mep = key.getMep();
                    final String execStatus = key.getExecStatus().toString();

                    this.shell.getPrintStream().println(
                            String.format(rowFormat, interfaceName == null ? "" : interfaceName,
                                    serviceName == null ? "" : serviceName, operation == null ? ""
                                            : operation, mep == null ? "" : mep,
                                    execStatus == null ? "" : execStatus, entry.getValue()[2]
                                            .longValue(), entry.getValue()[1].longValue(), entry
                                            .getValue()[0].longValue(), entry.getValue()[3]
                                            .longValue(), entry.getValue()[4].longValue(), entry
                                            .getValue()[5].longValue()));
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, String.format(
                    Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier));
        }
    }

}
