/**
 * Copyright (c) 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.container.transporter.local;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.exception.LocalTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.exception.LocalTransporterMonitoringServiceErrorException;

/**
 * The monitoring sub-function about delivered messages by the local transporter
 * of the Petals container
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DeliveredMessages implements MonitoringSubFunction {

    private final static String SUBFCT_NAME = "delivered-messages";

    private final static String SUBFCT_DESCRIPTION = "Delivered messages";

    /**
     * The option to get the list of endpoints
     */
    public static final String LIST_ENDPOINTS_LONG_OPTION = "list-endpoints";

    private static final Option LIST_ENDPOINTS_OPTION = OptionBuilder.isRequired(false).hasArgs(0)
            .withDescription("List the endpoints for which at least one message was delivered.")
            .withLongOpt(LIST_ENDPOINTS_LONG_OPTION).create();

    /**
     * The option to get the list of services
     */
    public static final String LIST_SERVICES_LONG_OPTION = "list-services";

    private static final Option LIST_SERVICES_OPTION = OptionBuilder.isRequired(false).hasArgs(0)
            .withDescription("List the services for which at least one message was delivered.")
            .withLongOpt(LIST_SERVICES_LONG_OPTION).create();

    /**
     * The option to get the list of interfaces
     */
    public static final String LIST_INTERFACES_LONG_OPTION = "list-interfaces";

    private static final Option LIST_INTERFACES_OPTION = OptionBuilder.isRequired(false).hasArgs(0)
            .withDescription("List the interfaces for which at least one message was delivered.")
            .withLongOpt(LIST_INTERFACES_LONG_OPTION).create();

    /**
     * The option to query delivered messages of endpoints
     */
    public static final String QUERY_ENDPOINTS_LONG_OPTION = "query-endpoints";

    private static final Option QUERY_ENDPOINTS_OPTION = OptionBuilder.isRequired(false)
            .hasOptionalArgs(2).withDescription("Query about endpoints")
            .withLongOpt(QUERY_ENDPOINTS_LONG_OPTION).create();

    /**
     * The option to query delivered messages of services
     */
    public static final String QUERY_SERVICES_LONG_OPTION = "query-services";

    private static final Option QUERY_SERVICES_OPTION = OptionBuilder.isRequired(false)
            .hasOptionalArgs(2).withDescription("Query about services")
            .withLongOpt(QUERY_SERVICES_LONG_OPTION).create();

    /**
     * The option to query delivered messages of interfaces
     */
    public static final String QUERY_INTERFACES_LONG_OPTION = "query-interfaces";

    private static final Option QUERY_INTERFACES_OPTION = OptionBuilder.isRequired(false)
            .hasOptionalArgs(2).withDescription("Query about interfaces")
            .withLongOpt(QUERY_INTERFACES_LONG_OPTION).create();

    /**
     * The option to get delivered message number consolidated by message
     * exchange status whatever endpoints, services and interfaces
     */
    public static final String MSG_STATUS_SHORT_OPTION = "m";

    private static final Option MSG_STATUS_OPTION = OptionBuilder.isRequired(false)
            .withDescription("Get consolidate values by message exchange status.")
            .create(MSG_STATUS_SHORT_OPTION);

    /**
     * The option to get the complete table of values
     */
    public static final String TABLE_SHORT_OPTION = "t";

    private static final Option TABLE_OPTION = OptionBuilder.isRequired(false)
            .withDescription("Get the complete table of values").create(TABLE_SHORT_OPTION);

    /**
     * Query attribute to get index name
     */
    public static final String QUERY_ATTR_NAME = "NAME";

    /**
     * Query attribute to get the number of delivered messages for message
     * exchange with status 'Active', required the index on which the query must
     * be run
     */
    public static final String QUERY_ATTR_MSG_STATUS_ACTIVE = "ACTIVE";

    /**
     * Query attribute to get the number of delivered messages for message
     * exchange with status 'Active', required the index on which the query must
     * be run
     */
    public static final String QUERY_ATTR_MSG_STATUS_ERROR = "ERROR";

    /**
     * Query attribute to get the number of delivered messages for message
     * exchange with status 'Active', required the index on which the query must
     * be run
     */
    public static final String QUERY_ATTR_MSG_STATUS_DONE = "DONE";

    /**
     * Index of the field 'interface' in the key of the metric 'delivered
     * message'
     */
    // TODO: Move the definition of indexes in a library common with
    // petals-kernel
    private final short INTERFACE_IDX = 0;

    /**
     * Index of the field 'service' in the key of the metric 'delivered message'
     */
    private final short SERVICE_IDX = 1;

    /**
     * Index of the field 'endpoint' in the key of the metric 'delivered
     * message'
     */
    private final short ENDPOINT_IDX = 2;

    /**
     * Index of the field 'message exchange status' in the key of the metric
     * 'delivered message'
     */
    private final short MSG_STATUS_IDX = 3;

    /**
     * Index of the field 'delivery status' in the key of the metric 'delivered
     * message'
     */
    private final short DELIVERY_STATUS_IDX = 4;

    private final static String HEADER_ITF = "Interface";

    private final static String HEADER_SVC = "Service";

    private final static String HEADER_EP = "Endpoint";

    private final static String HEADER_MSG_STATUS = "Msg status";

    private final static String HEADER_DELIVERY_STATUS = "Delivery Status";

    private final static String HEADER_VALUE = "Value";

    /**
     * The sub-function options
     */
    private Options options;

    /**
     * The shell in which the sub-function is running
     */
    private Shell shell;

    public DeliveredMessages() {
        super();
        this.createOptions();
    }

    // TODO: Options declaration must be reviewed to match:
    // [--list-endpoints] |
    // [--list-services] |
    // [--list-interfaces] |
    // [--query-endpoints [NAME] | [ACTIVE|DONE|ERROR <index>]] |
    // [--query-services [NAME] | [ACTIVE|DONE|ERROR <index>]] |
    // [--query-interfaces [NAME] | [ACTIVE|DONE|ERROR <index>]] |
    // [-m] | [-t]
    private void createOptions() {
        this.options = new Options();
        this.options.addOption(LIST_ENDPOINTS_OPTION);
        this.options.addOption(LIST_SERVICES_OPTION);
        this.options.addOption(LIST_INTERFACES_OPTION);
        this.options.addOption(QUERY_ENDPOINTS_OPTION);
        this.options.addOption(QUERY_SERVICES_OPTION);
        this.options.addOption(QUERY_INTERFACES_OPTION);
        this.options.addOption(MSG_STATUS_OPTION);
        this.options.addOption(TABLE_OPTION);
    }

    @Override
    public Options getOptions() {
        return this.options;
    }

    @Override
    public void resetOptions() {
        this.createOptions();
    }

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void setShell(final Shell shell) {
        this.shell = shell;
    }

    @Override
    public Shell getShell() {
        return this.shell;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionInvalidException,
            MonitoringSubFunctionException {

        final CommandLineParser parser = new PosixParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            if (cli.hasOption(LIST_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(LIST_SERVICES_LONG_OPTION)
                    && !cli.hasOption(LIST_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(QUERY_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(QUERY_SERVICES_LONG_OPTION)
                    && !cli.hasOption(QUERY_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(MSG_STATUS_SHORT_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listEndpoints(false, null);

            } else if (!cli.hasOption(LIST_ENDPOINTS_LONG_OPTION)
                    && cli.hasOption(LIST_SERVICES_LONG_OPTION)
                    && !cli.hasOption(LIST_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(QUERY_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(QUERY_SERVICES_LONG_OPTION)
                    && !cli.hasOption(QUERY_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(MSG_STATUS_SHORT_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listServices(false, null);

            } else if (!cli.hasOption(LIST_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(LIST_SERVICES_LONG_OPTION)
                    && cli.hasOption(LIST_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(QUERY_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(QUERY_SERVICES_LONG_OPTION)
                    && !cli.hasOption(QUERY_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(MSG_STATUS_SHORT_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listInterfaces(false, null);

            } else if (!cli.hasOption(LIST_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(LIST_SERVICES_LONG_OPTION)
                    && !cli.hasOption(LIST_INTERFACES_LONG_OPTION)
                    && cli.hasOption(QUERY_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(QUERY_SERVICES_LONG_OPTION)
                    && !cli.hasOption(QUERY_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(MSG_STATUS_SHORT_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listEndpoints(true, cli.getOptionValues(QUERY_ENDPOINTS_LONG_OPTION));

            } else if (!cli.hasOption(LIST_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(LIST_SERVICES_LONG_OPTION)
                    && !cli.hasOption(LIST_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(QUERY_ENDPOINTS_LONG_OPTION)
                    && cli.hasOption(QUERY_SERVICES_LONG_OPTION)
                    && !cli.hasOption(QUERY_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(MSG_STATUS_SHORT_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listServices(true, cli.getOptionValues(QUERY_ENDPOINTS_LONG_OPTION));

            } else if (!cli.hasOption(LIST_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(LIST_SERVICES_LONG_OPTION)
                    && !cli.hasOption(LIST_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(QUERY_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(QUERY_SERVICES_LONG_OPTION)
                    && cli.hasOption(QUERY_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(MSG_STATUS_SHORT_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listInterfaces(true, cli.getOptionValues(QUERY_ENDPOINTS_LONG_OPTION));

            } else if (!cli.hasOption(LIST_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(LIST_SERVICES_LONG_OPTION)
                    && !cli.hasOption(LIST_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(QUERY_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(QUERY_SERVICES_LONG_OPTION)
                    && !cli.hasOption(QUERY_INTERFACES_LONG_OPTION)
                    && cli.hasOption(MSG_STATUS_SHORT_OPTION) && !cli.hasOption(TABLE_SHORT_OPTION)) {

                final long active = this.getConsolidatedValues(null, null, null, "Active", null);
                final long done = this.getConsolidatedValues(null, null, null, "Done", null);
                final long error = this.getConsolidatedValues(null, null, null, "Error", null);

                this.shell.getPrintStream().println(
                        String.format("Active:%d Done:%d Error:%d", active, done, error));

            } else if (!cli.hasOption(LIST_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(LIST_SERVICES_LONG_OPTION)
                    && !cli.hasOption(LIST_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(QUERY_ENDPOINTS_LONG_OPTION)
                    && !cli.hasOption(QUERY_SERVICES_LONG_OPTION)
                    && !cli.hasOption(QUERY_INTERFACES_LONG_OPTION)
                    && !cli.hasOption(MSG_STATUS_SHORT_OPTION) && cli.hasOption(TABLE_SHORT_OPTION)) {

                this.fullTable();

            } else {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        }
    }

    /**
     * <p>
     * List endpoints for which a metric 'delivered messages' is available
     * </p>
     * .
     * <p>
     * If no query is asked, the number of delivered messages is not returned.
     * Otherwise, the number of delivered messages is returned accordingly to
     * the message exchange status (1st query argument) and endpoint (2nd query
     * argument).
     * </p>
     * 
     * @param hasQuery
     *            A query is asked
     * @param queryArgs
     *            Arguments of the query
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the metric value.
     */
    private void listEndpoints(final boolean hasQuery, final String[] queryArgs)
            throws MonitoringSubFunctionException {

        try {

            final JMXClient client = JMXClientConnection.getJMXClient();
            final Map<String[], Long> values = client.getLocalTransporterMonitoringServiceClient()
                    .getDeliveredMessage();

            if (hasQuery) {
                this.query(values, queryArgs, ENDPOINT_IDX, QUERY_ENDPOINTS_OPTION);
            } else {
                final List<String> endpoints = new ArrayList<String>();
                for (final String[] key : values.keySet()) {
                    if (!endpoints.contains(key[ENDPOINT_IDX])) {
                        endpoints.add(key[ENDPOINT_IDX]);
                    }
                }

                for (final String endpoint : endpoints) {
                    this.shell.getPrintStream().println(endpoint);
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

    /**
     * List services for which a metric 'delivered messages' is available
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the metric value.
     */
    private void listServices(final boolean hasQuery, final String[] queryArgs)
            throws MonitoringSubFunctionException {

        try {
            final JMXClient client = JMXClientConnection.getJMXClient();
            final Map<String[], Long> values = client.getLocalTransporterMonitoringServiceClient()
                    .getDeliveredMessage();
            if (hasQuery) {
                this.query(values, queryArgs, SERVICE_IDX, QUERY_SERVICES_OPTION);
            } else {
                final List<String> services = new ArrayList<String>();
                for (final String[] key : values.keySet()) {
                    if (!services.contains(key[SERVICE_IDX])) {
                        services.add(key[SERVICE_IDX]);
                    }
                }

                for (final String service : services) {
                    this.shell.getPrintStream().println(service);
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

    /**
     * List interfaces for which a metric 'delivered messages' is available
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the metric value.
     */
    private void listInterfaces(final boolean hasQuery, final String[] queryArgs)
            throws MonitoringSubFunctionException {

        try {
            final JMXClient client = JMXClientConnection.getJMXClient();
            final Map<String[], Long> values = client.getLocalTransporterMonitoringServiceClient()
                    .getDeliveredMessage();
            if (hasQuery) {
                this.query(values, queryArgs, INTERFACE_IDX, QUERY_INTERFACES_OPTION);
            } else {
                final List<String> interfaces = new ArrayList<String>();
                for (final String[] key : values.keySet()) {
                    if (!interfaces.contains(key[INTERFACE_IDX])) {
                        interfaces.add(key[INTERFACE_IDX]);
                    }
                }

                for (final String itf : interfaces) {
                    this.shell.getPrintStream().println(itf);
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

    private void query(final Map<String[], Long> values, final String[] queryArgs,
            final short indexOfIndex, final Option queryOption)
            throws MonitoringSubFunctionException {

        if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null
                || queryArgs[0].isEmpty()) {
            throw new MonitoringSubFunctionMissingArgumentException(this, queryOption);
        } else {
            final String index = queryArgs.length == 2 ? queryArgs[1] : null;
            final Map<String, Long> valuesByStatus;
            // TODO: Use constants defined in a library common with
            // petals-kernel and replace "Active", "Done" and "Error"
            if (QUERY_ATTR_NAME.equals(queryArgs[0])) {
                final List<String> endpoints = new ArrayList<String>();
                for (final String[] key : values.keySet()) {
                    if (!endpoints.contains(key[ENDPOINT_IDX])) {
                        endpoints.add(key[ENDPOINT_IDX]);
                    }
                }

                for (final String endpoint : endpoints) {
                    this.shell.getPrintStream().println(String.format("%s!%s", endpoint, endpoint));
                }
            } else {
                if (QUERY_ATTR_MSG_STATUS_ACTIVE.equals(queryArgs[0])) {
                    valuesByStatus = this.agregate(values, "Active");
                } else if (QUERY_ATTR_MSG_STATUS_DONE.equals(queryArgs[0])) {
                    valuesByStatus = this.agregate(values, "Done");
                } else if (QUERY_ATTR_MSG_STATUS_ERROR.equals(queryArgs[0])) {
                    valuesByStatus = this.agregate(values, "Error");
                } else {
                    throw new MonitoringSubFunctionInvalidArgumentException(this, queryOption,
                            queryArgs[0]);
                }

                if (index != null) {
                    final Long value = valuesByStatus.get(index);
                    if (value == null) {
                        this.shell.getPrintStream().println(String.format("%d", 0));
                    } else {
                        this.shell.getPrintStream().println(String.format("%s", value));
                    }
                } else {
                    for (final Map.Entry<String, Long> entry : valuesByStatus.entrySet()) {
                        this.shell.getPrintStream().println(
                                String.format("%s!%d", entry.getKey(), entry.getValue()));
                    }
                }
            }
        }
    }

    private Map<String, Long> agregate(final Map<String[], Long> values, final String status) {
        final Map<String, Long> results = new HashMap<String, Long>();
        for (final Map.Entry<String[], Long> entry : values.entrySet()) {
            final String[] key = entry.getKey();
            if (key[MSG_STATUS_IDX].equals(status)) {
                final Long agregatedValue = results.get(key[ENDPOINT_IDX]);
                results.put(key[ENDPOINT_IDX], agregatedValue == null ? entry.getValue()
                        : agregatedValue + entry.getValue());
            }
        }
        return results;
    }

    /**
     * Print the full table
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     */
    private void fullTable() throws MonitoringSubFunctionException {

        try {
            final JMXClient client = JMXClientConnection.getJMXClient();
            final Map<String[], Long> values = client.getLocalTransporterMonitoringServiceClient()
                    .getDeliveredMessage();

            int maxSizeItf = 0;
            int maxSizeSvc = 0;
            int maxSizeEp = 0;
            int maxSizeMsgStatus = 0;
            int maxSizeDeliveryStatus = 0;
            int maxSizeValue = 0;
            for (final Map.Entry<String[], Long> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                maxSizeItf = Math.max(maxSizeItf, key[INTERFACE_IDX] == null ? 0
                        : key[INTERFACE_IDX].length());
                maxSizeSvc = Math.max(maxSizeSvc,
                        key[SERVICE_IDX] == null ? 0 : key[SERVICE_IDX].length());
                maxSizeEp = Math.max(maxSizeEp,
                        key[ENDPOINT_IDX] == null ? 0 : key[ENDPOINT_IDX].length());
                maxSizeMsgStatus = Math.max(maxSizeMsgStatus, key[MSG_STATUS_IDX] == null ? 0
                        : key[MSG_STATUS_IDX].length());
                maxSizeDeliveryStatus = Math.max(maxSizeDeliveryStatus,
                        key[DELIVERY_STATUS_IDX] == null ? 0 : key[DELIVERY_STATUS_IDX].length());
                maxSizeValue = Math.max(maxSizeValue, entry.getValue().toString().length());
            }

            maxSizeItf = Math.max(maxSizeItf, HEADER_ITF.length());
            maxSizeSvc = Math.max(maxSizeSvc, HEADER_SVC.length());
            maxSizeEp = Math.max(maxSizeEp, HEADER_EP.length());
            maxSizeMsgStatus = Math.max(maxSizeMsgStatus, HEADER_MSG_STATUS.length());
            maxSizeDeliveryStatus = Math
                    .max(maxSizeDeliveryStatus, HEADER_DELIVERY_STATUS.length());
            maxSizeValue = Math.max(maxSizeValue, HEADER_VALUE.length());

            final String rowHeaderFormat = "| %-" + maxSizeItf + "s | %-" + maxSizeSvc + "s | %-"
                    + maxSizeEp + "s | %-" + maxSizeMsgStatus + "s | %-" + maxSizeDeliveryStatus
                    + "s || %-" + maxSizeValue + "s |";
            final String header = String.format(rowHeaderFormat, HEADER_ITF, HEADER_SVC, HEADER_EP,
                    HEADER_MSG_STATUS, HEADER_DELIVERY_STATUS, HEADER_VALUE);
            this.shell.getPrintStream().println(header);
            for (int i = 0; i < header.length(); i++) {
                this.shell.getPrintStream().append('-');
            }
            this.shell.getPrintStream().append('\n');

            final String rowFormat = "| %-" + maxSizeItf + "s | %-" + maxSizeSvc + "s | %-"
                    + maxSizeEp + "s | %-" + maxSizeMsgStatus + "s | %-" + maxSizeDeliveryStatus
                    + "s || %" + maxSizeValue + "d |";
            for (final Map.Entry<String[], Long> entry : values.entrySet()) {

                final String[] key = entry.getKey();

                this.shell.getPrintStream().println(
                        String.format(rowFormat, key[INTERFACE_IDX] == null ? ""
                                : key[INTERFACE_IDX], key[SERVICE_IDX] == null ? ""
                                : key[SERVICE_IDX], key[ENDPOINT_IDX] == null ? ""
                                : key[ENDPOINT_IDX], key[MSG_STATUS_IDX] == null ? ""
                                : key[MSG_STATUS_IDX], key[DELIVERY_STATUS_IDX] == null ? ""
                                : key[DELIVERY_STATUS_IDX], entry.getValue().longValue()));
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

    /**
     * Get the consolidated value matching provided regexp
     * 
     * @param interfaceRegexp
     *            The regexp of interfaces for which the consolidation must be
     *            done
     * @param serviceRegexp
     *            The regexp of services for which the consolidation must be
     *            done
     * @param endpointRegexp
     *            The regexp of endpoints for which the consolidation must be
     *            done
     * @param msgExStatusRegexp
     *            Message exchange status filter. If <code>null</code>, no
     *            filter applied.
     * @param deliveryStatusRegexp
     *            Message exchange delivery status filter. If <code>null</code>,
     *            no filter applied.
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     */
    private long getConsolidatedValues(final String interfaceRegexp, final String serviceRegexp,
            final String endpointRegexp, final String msgExStatusRegexp,
            final String deliveryStatusRegexp) throws MonitoringSubFunctionException {

        try {

            final Pattern interfaceNamePattern;
            try {
                interfaceNamePattern = Pattern.compile(interfaceRegexp == null ? ".*"
                        : interfaceRegexp);
            } catch (final PatternSyntaxException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        "Invalid regexp: '%s'", interfaceRegexp), e);
            }

            final Pattern serviceNamePattern;
            try {
                serviceNamePattern = Pattern.compile(serviceRegexp == null ? ".*" : serviceRegexp);
            } catch (final PatternSyntaxException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        "Invalid regexp: '%s'", serviceRegexp), e);
            }

            final Pattern endpointNamePattern;
            try {
                endpointNamePattern = Pattern.compile(endpointRegexp == null ? ".*"
                        : endpointRegexp);
            } catch (final PatternSyntaxException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        "Invalid regexp: '%s'", endpointRegexp), e);
            }

            final Pattern msgExStatusPattern;
            try {
                msgExStatusPattern = Pattern.compile(msgExStatusRegexp == null ? ".*"
                        : msgExStatusRegexp);
            } catch (final PatternSyntaxException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        "Invalid regexp: '%s'", msgExStatusRegexp), e);
            }

            final Pattern deliveryStatusPattern;
            try {
                deliveryStatusPattern = Pattern.compile(deliveryStatusRegexp == null ? ".*"
                        : deliveryStatusRegexp);
            } catch (final PatternSyntaxException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        "Invalid regexp: '%s'", deliveryStatusRegexp), e);
            }

            final JMXClient client = JMXClientConnection.getJMXClient();
            final Map<String[], Long> values = client.getLocalTransporterMonitoringServiceClient()
                    .getDeliveredMessage();

            long consolidatedValue = 0;
            for (final Map.Entry<String[], Long> entry : values.entrySet()) {

                final String[] key = entry.getKey();
                final Matcher interfaceNameMatcher = interfaceNamePattern
                        .matcher(key[INTERFACE_IDX] == null ? "" : key[INTERFACE_IDX]);
                final Matcher serviceNameMatcher = serviceNamePattern
                        .matcher(key[SERVICE_IDX] == null ? "" : key[SERVICE_IDX]);
                final Matcher endpointNameMatcher = endpointNamePattern
                        .matcher(key[ENDPOINT_IDX] == null ? "" : key[ENDPOINT_IDX]);
                final Matcher msgExStatusMatcher = msgExStatusPattern
                        .matcher(key[MSG_STATUS_IDX] == null ? "" : key[MSG_STATUS_IDX]);
                final Matcher deliveryStatusMatcher = deliveryStatusPattern
                        .matcher(key[DELIVERY_STATUS_IDX] == null ? "" : key[DELIVERY_STATUS_IDX]);

                if (interfaceNameMatcher.matches() && serviceNameMatcher.matches()
                        && endpointNameMatcher.matches() && msgExStatusMatcher.matches()
                        && deliveryStatusMatcher.matches()) {
                    consolidatedValue += entry.getValue().longValue();
                }
            }

            return consolidatedValue;

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

}
