/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.util.SimpleShell;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvokationKey;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvokationStatus;
import org.ow2.petals.jmx.api.mock.PetalsJmxApiFactoryMock;
import org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock;

/**
 * Unit tests of {@link ServiceProviderInvokationsResponseTimes}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ServiceProviderInvokationsResponseTimesTest extends AbstractTestCase {

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        try {
            monitoringSubFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        try {
            monitoringSubFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided without argument</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_02() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        try {
            monitoringSubFunction.execute(new String[] { "-"
                    + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option",
                    ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION, e
                            .getOption().getOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_03() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_04() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvokationsResponseTimes.LIST_OPERATIONS_LONG_OPTION,
                    "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_05() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvokationsResponseTimes.LIST_OPERATIONS_LONG_OPTION,
                    "--" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                    ServiceProviderInvokationsResponseTimes.QUERY_ATTR_NAME });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_06() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvokationsResponseTimes.LIST_OPERATIONS_LONG_OPTION,
                    "-" + ServiceProviderInvokationsResponseTimes.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_07() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("The option having a missing argument is not the expected one",
                    ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION, e
                            .getOption().getLongOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with an invalid argument.
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the invalid argument.
     * </p>
     */
    @Test
    public void testExecute_Error_08() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        final String invalidArg = "invalid-arg";
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                    invalidArg });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("The option having an invalid argument is not the expected one",
                    ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION, e
                            .getOption().getLongOpt());
            assertEquals("The invalid value is not the expected one", invalidArg, e.getValue());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_09() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                    ServiceProviderInvokationsResponseTimes.QUERY_ATTR_NAME,
                    "-" + ServiceProviderInvokationsResponseTimes.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '-t' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_10() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        try {
            monitoringSubFunction
                    .execute(new String[] {
                            "-"
                                    + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "-" + ServiceProviderInvokationsResponseTimes.TABLE_SHORT_OPTION,
                            "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--list-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException, ConnectionErrorException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + ServiceProviderInvokationsResponseTimes.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_01() throws MonitoringSubFunctionException, ConnectionErrorException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "--" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException, ConnectionErrorException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + ServiceProviderInvokationsResponseTimes.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_03() throws MonitoringSubFunctionException, ConnectionErrorException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + ServiceProviderInvokationsResponseTimes.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_04() throws MonitoringSubFunctionException, ConnectionErrorException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "--" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_05() throws MonitoringSubFunctionException, ConnectionErrorException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + ServiceProviderInvokationsResponseTimes.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing
     * ws-request response times,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_06() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = null;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.LIST_OPERATIONS_LONG_OPTION });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing
     * ws-request response times,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_07() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = null;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_NAME });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing
     * ws-request response times,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_08() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = null;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.TABLE_SHORT_OPTION });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        final String header = brOutput.readLine();
        assertTrue("No header", header != null && !header.isEmpty());
        final String sepLine = brOutput.readLine();
        assertTrue("No line", sepLine != null && !sepLine.isEmpty());
        final String end = brOutput.readLine();
        assertNull("Extra character displayed: [" + end + "]", end);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-list-operations' is provided without argument,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_09() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long[] serviceProviderInvokationsResponseTimes1 = new Long[] { Long.valueOf(789),
                Long.valueOf(789), Long.valueOf(789), Long.valueOf(789), Long.valueOf(789),
                Long.valueOf(789) };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long[] serviceProviderInvokationResponseTimes2 = new Long[] { Long.valueOf(10),
                Long.valueOf(10), Long.valueOf(10), Long.valueOf(10), Long.valueOf(10),
                Long.valueOf(10) };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationInterfaceName1, serviceProviderInvokationServiceName1,
                serviceProviderInvokationOperation1, "clientIpAddress",
                ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsResponseTimes1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES
                .put(new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        "clientIpAddress", ServiceProviderInvokationStatus.SUCCEEDED),
                        serviceProviderInvokationResponseTimes2);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.LIST_OPERATIONS_LONG_OPTION });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            boolean operationFound = false;
            for (final Map.Entry<ServiceProviderInvokationKey, Long[]> entry : SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES
                    .entrySet()) {
                final ServiceProviderInvokationKey key = entry.getKey();
                if (line.equals(key.getOperation() + "@" + key.getServiceName() + "@"
                        + key.getInterfaceName())) {
                    operationFound = true;
                    nbOperations++;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
        }
        assertEquals("unexpected operation number",
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES.size(), nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-query-operations' is provided with the argument "NAME",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_10() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                Long.valueOf(789), Long.valueOf(789), Long.valueOf(789), Long.valueOf(789),
                Long.valueOf(789), Long.valueOf(789) };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                Long.valueOf(10), Long.valueOf(10), Long.valueOf(10), Long.valueOf(10),
                Long.valueOf(10), Long.valueOf(10) };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] { Long.valueOf(56),
                Long.valueOf(56), Long.valueOf(56), Long.valueOf(56), Long.valueOf(56),
                Long.valueOf(56) };
        final Long[] serviceProviderInvokationResponseTimesFault3 = new Long[] { Long.valueOf(34),
                Long.valueOf(34), Long.valueOf(34), Long.valueOf(34), Long.valueOf(34),
                Long.valueOf(34) };
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] { Long.valueOf(9),
                Long.valueOf(9), Long.valueOf(9), Long.valueOf(9), Long.valueOf(9), Long.valueOf(9) };
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] { Long.valueOf(7),
                Long.valueOf(7), Long.valueOf(7), Long.valueOf(7), Long.valueOf(7), Long.valueOf(7) };

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationInterfaceName1, serviceProviderInvokationServiceName1,
                serviceProviderInvokationOperation1, null,
                ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationInterfaceName1, serviceProviderInvokationServiceName1,
                serviceProviderInvokationOperation1, null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationInterfaceName1, serviceProviderInvokationServiceName1,
                serviceProviderInvokationOperation1, null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationInterfaceName2, serviceProviderInvokationServiceName2,
                serviceProviderInvokationOperation2, null,
                ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationInterfaceName2, serviceProviderInvokationServiceName2,
                serviceProviderInvokationOperation2, null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault3);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationInterfaceName2, serviceProviderInvokationServiceName2,
                serviceProviderInvokationOperation2, null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1, null);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2, null);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_NAME });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<ServiceProviderInvokationKey, Long[]> entry : SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES
                    .entrySet()) {
                final ServiceProviderInvokationKey key = entry.getKey();
                if (operationNameReturned.equals(key.getOperation() + "@" + key.getServiceName()
                        + "@" + key.getInterfaceName())) {
                    operationFound = true;
                    assertEquals("Unexpected name value", operationNameReturned, value);
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMin() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_min);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesSucceeded2_min);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededAvg() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_avg);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesSucceeded2_avg);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMax() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_max);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesSucceeded2_max);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded10P() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_10p);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesSucceeded2_10p);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_10P });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded50P() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf1-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf1-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1, serviceProviderInvokationOperation1,
                        null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2, serviceProviderInvokationOperation2,
                        null, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_50p);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesSucceeded2_50p);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_50P });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded90P() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf1-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf1-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_90p);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesSucceeded2_90p);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_90P });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMin() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_min);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesFault2_min);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_MIN });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultAvg() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_avg);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesFault2_avg);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_AVG });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMax() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_max);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesFault2_max);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_MAX });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault10P() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_10p);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesFault2_10p);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_10P });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute__Query_Fault50P() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_50p);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesFault2_50p);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_50P });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault90P() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_90p);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesFault2_90p);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_90P });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMin() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_min);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesError2_min);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_MIN });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorAvg() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_avg);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesError2_avg);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_AVG });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMax() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_max);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesError2_max);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_MAX });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error10P() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_10p);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesError2_10p);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_10P });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error50P() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_50p);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesError2_50p);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_50P });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error90P() throws MonitoringSubFunctionException,
            ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_90p);
        expectedResults.put(serviceProviderInvokationOperation2 + "@"
                + serviceProviderInvokationServiceName2 + "@"
                + serviceProviderInvokationInterfaceName2,
                serviceProviderInvokationResponseTimesError2_90p);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_90P });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMin_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_min);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_avg);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMax_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_max);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_10p);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_10P,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_50p);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_50P,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded90P_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesSucceeded1_90p);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_90P,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMin_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_min);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_MIN, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_avg);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_AVG, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMax_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_max);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_MAX, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_10p);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_10P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_50p);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_50P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault90P_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesFault1_90p);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_90P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMin_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_min);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_MIN, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_avg);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_AVG, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMax_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_max);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_MAX, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_10p);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_10P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_50p);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_50P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error90P_With_Valid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1,
                serviceProviderInvokationResponseTimesError1_90p);

        final String queriedOperation = serviceProviderInvokationOperation1 + "@"
                + serviceProviderInvokationServiceName1 + "@"
                + serviceProviderInvokationInterfaceName1;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_90P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_10P,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_50P,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction
                .execute(new String[] {
                        "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvokationsResponseTimes.QUERY_ATTR_SUCCEEDED_90P,
                        queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_MIN, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_AVG, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_MAX, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_10P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_50P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_FAULT_90P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_MIN, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_AVG, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_MAX, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_10P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_50P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsResponseTimes();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final Long serviceProviderInvokationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvokationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvokationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvokationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvokationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvokationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvokationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded1_max,
                serviceProviderInvokationResponseTimesSucceeded1_avg,
                serviceProviderInvokationResponseTimesSucceeded1_min,
                serviceProviderInvokationResponseTimesSucceeded1_10p,
                serviceProviderInvokationResponseTimesSucceeded1_50p,
                serviceProviderInvokationResponseTimesSucceeded1_90p };
        final String serviceProviderInvokationInterfaceName1 = "itf-1";
        final String serviceProviderInvokationServiceName1 = "svc-1";
        final String serviceProviderInvokationOperation1 = "operation1";
        final Long serviceProviderInvokationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvokationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvokationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvokationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvokationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvokationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvokationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvokationResponseTimesSucceeded2_max,
                serviceProviderInvokationResponseTimesSucceeded2_avg,
                serviceProviderInvokationResponseTimesSucceeded2_min,
                serviceProviderInvokationResponseTimesSucceeded2_10p,
                serviceProviderInvokationResponseTimesSucceeded2_50p,
                serviceProviderInvokationResponseTimesSucceeded2_90p };
        final String serviceProviderInvokationInterfaceName2 = "itf-2";
        final String serviceProviderInvokationServiceName2 = "svc-2";
        final String serviceProviderInvokationOperation2 = "operation2";
        final Long serviceProviderInvokationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvokationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvokationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvokationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvokationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvokationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvokationResponseTimesFault1 = new Long[] {
                serviceProviderInvokationResponseTimesFault1_max,
                serviceProviderInvokationResponseTimesFault1_avg,
                serviceProviderInvokationResponseTimesFault1_min,
                serviceProviderInvokationResponseTimesFault1_10p,
                serviceProviderInvokationResponseTimesFault1_50p,
                serviceProviderInvokationResponseTimesFault1_90p };
        final Long serviceProviderInvokationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvokationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvokationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvokationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvokationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvokationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvokationResponseTimesFault2 = new Long[] {
                serviceProviderInvokationResponseTimesFault2_max,
                serviceProviderInvokationResponseTimesFault2_avg,
                serviceProviderInvokationResponseTimesFault2_min,
                serviceProviderInvokationResponseTimesFault2_10p,
                serviceProviderInvokationResponseTimesFault2_50p,
                serviceProviderInvokationResponseTimesFault2_90p };
        final Long serviceProviderInvokationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvokationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvokationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvokationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvokationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvokationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvokationResponseTimesError1 = new Long[] {
                serviceProviderInvokationResponseTimesError1_max,
                serviceProviderInvokationResponseTimesError1_avg,
                serviceProviderInvokationResponseTimesError1_min,
                serviceProviderInvokationResponseTimesError1_10p,
                serviceProviderInvokationResponseTimesError1_50p,
                serviceProviderInvokationResponseTimesError1_90p };
        final Long serviceProviderInvokationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvokationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvokationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvokationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvokationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvokationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvokationResponseTimesError2 = new Long[] {
                serviceProviderInvokationResponseTimesError2_max,
                serviceProviderInvokationResponseTimesError2_avg,
                serviceProviderInvokationResponseTimesError2_min,
                serviceProviderInvokationResponseTimesError2_10p,
                serviceProviderInvokationResponseTimesError2_50p,
                serviceProviderInvokationResponseTimesError2_90p };

        // Service provider invokation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<ServiceProviderInvokationKey, Long[]>();
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault1);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName1,
                        serviceProviderInvokationServiceName1,
                        serviceProviderInvokationOperation1, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError1);

        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationResponseTimesFault2);
        SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS.put(
                new ServiceProviderInvokationKey(serviceProviderInvokationInterfaceName2,
                        serviceProviderInvokationServiceName2,
                        serviceProviderInvokationOperation2, null,
                        ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsResponseTimes.QUERY_ATTR_ERROR_90P, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }
}
